import { useEffect, useState } from "preact/hooks";
import { Modal } from "../components/modal.tsx";
import { LabelledHr } from "../components/labelledHr.tsx";
import { PacksList } from "../components/packsList.tsx";
import { PackInfo } from "../components/packInfo.tsx";
import { useAtom } from "jotai";
import { namespaceAtom } from "../atoms/namespace.ts";
import { Route, Routes } from "react-router-dom";
import { Selector } from "../components/editor/selector.tsx";
import { NamespaceModal } from "../components/editor/namespaceModal.tsx";

export const Editor = () => {
  const [packName, setPackName] = useState("");
  const [loading, setLoading] = useState(true);
  const [namespaces, setNamespaces] = useState<string[]>([]);
  const [namespace, setNamespace] = useAtom(namespaceAtom);

  const fetchNamespaces = async () => {
    const res = await fetch("/api/pack/namespaces");
    const json = await res.json();
    setNamespaces(json);
  };

  useEffect(() => {
    document.title = "BearMetal Packer";
    fetch("/api/pack").then((res) => res.json()).then((json) => {
      setPackName(json.packName);
      setLoading(false);
    });
    fetchNamespaces();
  }, []);

  const setPackNameThing = async (event: SubmitEvent) => {
    setLoading(true);
    const res = await fetch("/api/pack", {
      method: "POST",
      body: new FormData(event.target as HTMLFormElement),
    });

    if (res.status === 200) {
      document.title = packName;
      setPackName(packName);
    }
    setLoading(false);
  };

  if (!packName) {
    return (
      <Modal>
        {loading
          ? <p>Just a sec, trying to put the cats back in the bag.</p>
          : (
            <>
              <form
                method="POST"
                onSubmit={setPackNameThing}
              >
                <label class="w-full">Set pack name</label>
                <div class="flex gap-2">
                  <input type="text" name="packName" />
                  <button type="submit">Set</button>
                </div>
              </form>
              <LabelledHr>OR</LabelledHr>
              <PacksList setPackName={setPackName} />
            </>
          )}
      </Modal>
    );
  }

  const [showNamespaceModal, setShowNamespaceModal] = useState(false);

  return (
    <div class="flex h-full">
      <div class="w-1/4 p-4 bg-mixed-400">
        <div class="text-center">
          <h1 class="text-2xl">BearMetalPacker</h1>
          <PackInfo packName={packName} />
        </div>
        <LabelledHr>Namespaces</LabelledHr>
        <div>
          <ul class="w-full">
            {namespaces.map((namespace) => (
              <li
                class="text-lg cursor-pointer even:bg-black/5"
                onClick={() => setNamespace(namespace)}
              >
                {namespace}
              </li>
            ))}
            <li class="mt-4">
              <button onClick={() => setShowNamespaceModal(true)}>
                New Namespace
              </button>
            </li>
          </ul>
          {showNamespaceModal && (
            <NamespaceModal
              close={() => {
                setShowNamespaceModal(false);
                fetchNamespaces();
              }}
            />
          )}
        </div>
      </div>
      <div>
        {!namespace ? <div>No namespace set</div> : (
          <div>
            <h3 class="text-lg">Namespace: {namespace}</h3>
            <Routes>
              <Route path="/editor" Component={Selector} />
            </Routes>
          </div>
        )}
      </div>
    </div>
  );
};
